import 'dart:math';

import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

class TextFieldWidget extends StatefulWidget {
  final Text? title;

  final void Function(String?)? saveValue;
  final String? Function()? getValue;

  final FieldValidation? validation;
  final FocusNode? focusNode;
  final FocusNode? nextFocusNode;
  final void Function(String)? onSubmit;
  final TextInputType? keyboardType;
  final TextCapitalization? textCapitalization;
  final bool multiLine;
  final int? maxLines;
  final bool? isEditable;
  final bool? isDisabled;

  final HighlightController? highlightController;

  const TextFieldWidget({
    Key? key,
    this.saveValue,
    this.title,
    required this.getValue,
    this.validation,
    this.focusNode,
    this.nextFocusNode,
    this.onSubmit,
    this.keyboardType,
    this.multiLine = false,
    this.maxLines = 8,
    required this.isEditable,
    this.isDisabled = false,
    this.highlightController,
    this.textCapitalization,
  })  : assert(getValue != null),
        assert(!(isEditable ?? true) || saveValue != null),
        super(key: key);

  @override
  _TextFieldWidgetState createState() => _TextFieldWidgetState();
}

class _TextFieldWidgetState extends State<TextFieldWidget> {
  String? errorMessage;
  FocusNode? validationFocusNode;

  ScrollController _scrollController = ScrollController();

  @override
  void initState() {
    super.initState();

    widget.validation?.connectToField(
        onValidate: (errorMessage) {
          if (mounted) setState(() => this.errorMessage = errorMessage);
        },
        focusNode: widget.focusNode ?? (validationFocusNode = FocusNode()));
  }

  @override
  Widget build(BuildContext context) {
    var isFirst = ItemPosition.of(context)?.isFirst ?? false;
    var isLast = ItemPosition.of(context)?.isLast ?? false;
    var hasSeparationLine =
        ItemPosition.of(context)?.hasSeparationLine ?? false;

    var text = widget.getValue?.call() ?? "";
    var controller = TextEditingController()
      ..value = TextEditingValue(
          text: text,
          selection: TextSelection(
              baseOffset: text.length, extentOffset: text.length));
    controller.addListener(() => widget.saveValue
        ?.call(controller.text.trim() == "" ? null : controller.text.trim()));

    var inputBorder = hasSeparationLine
        ? UnderlineInputBorder(
            borderSide:
                BorderSide(color: DefaultColors.backgroundOutline, width: 1))
        : InputBorder.none;
    var focusedBorder = UnderlineInputBorder(
        borderSide: BorderSide(
      color: DefaultColors.accent,
      width: 2,
    ));

    return ItemPosition(
      hasSeparationLine: false,
      isFirst: isFirst,
      isLast: isLast,
      child: DefaultFieldWidget(
        title: widget.title,
        highlightController: widget.highlightController,
        childPadding:
            const EdgeInsets.only(left: 30, right: 30, bottom: 8, top: 3),
        child: Scrollbar(
          controller: _scrollController,
          thumbVisibility: widget.multiLine,
          child: TextField(
            style: TextStyles.textStyleSize(17).copyWith(
              color: widget.isDisabled!
                  ? DefaultColors.textLight
                  : DefaultColors.text,
            ),
            scrollController: _scrollController,
            decoration: InputDecoration(
              contentPadding: EdgeInsets.only(top: 2, bottom: 10),
              isDense: true,
              enabledBorder: inputBorder,
              disabledBorder: inputBorder,
              focusedBorder: (widget.isEditable! && !widget.isDisabled!)
                  ? focusedBorder
                  : inputBorder,
              errorText: errorMessage,
            ),
            controller: controller,
            cursorColor: DefaultColors.accentLight,
            enabled: true,
            readOnly: !widget.isEditable! || widget.isDisabled!,
            keyboardType: widget.keyboardType,
            maxLines: widget.multiLine ? widget.maxLines : 1,
            minLines: widget.multiLine
                ? min(max(1, controller.text.split("\n").length),
                    (widget.maxLines ?? 100))
                : 1,
            textInputAction: widget.multiLine
                ? TextInputAction.newline
                : widget.nextFocusNode == null
                    ? TextInputAction.done
                    : TextInputAction.next,
            textCapitalization:
                widget.textCapitalization ?? TextCapitalization.sentences,
            focusNode: widget.focusNode ?? validationFocusNode,
            onSubmitted: (value) {
              if (widget.multiLine) return;
              if (widget.nextFocusNode != null)
                FocusScope.of(context).requestFocus(widget.nextFocusNode);
              widget.onSubmit?.call(value);
            },
          ),
        ),
      ),
    );
  }
}
